package org.rcisoft.business.bdiscuss.service.impl;

//import com.netflix.discovery.converters.Auto;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.bdiscuss.dto.BDiscussDto;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.enums.LessonTypeEnum;
import org.rcisoft.business.blesson.service.BLessonService;
import org.rcisoft.business.bmessage.dao.BMessageRepository;
import org.rcisoft.business.bmessage.entity.BMessage;
import org.rcisoft.business.bnotice.entity.BNotice;
import org.rcisoft.business.bnotice.service.BNoticeService;
import org.rcisoft.common.model.InfoTypeEnum;
import org.rcisoft.common.util.feignDto.MTUserGetsReqDTO;
import org.rcisoft.common.util.feignDto.MTUserInfoRspDTO;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.common.util.outClient.MTMicroappApiRequestClient;
import org.rcisoft.common.util.outClient.MTNotificationApiRequestClient;
import org.rcisoft.core.enums.MessageEnum;
import org.rcisoft.core.enums.UrlTypeEnum;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bdiscuss.dao.BDiscussRepository;
import org.rcisoft.business.bdiscuss.entity.BDiscuss;
import org.rcisoft.business.bdiscuss.service.BDiscussService;


import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.*;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2019-9-25 9:03:09.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BDiscussServiceImpl implements BDiscussService {

    @Autowired
    private BDiscussRepository bDiscussRepository;

    @Autowired
    MTCotactApiRequestClient mtCotactApiRequestClient;

    @Autowired
    BLessonRepository bLessonRepository;

    @Autowired
    MTNotificationApiRequestClient notificationApiRequestClient;

    @Autowired
    SysUserService sysUserServiceImpl;

    @Autowired
    BLessonRepository lessonRepository;

    @Autowired
    MTNotificationApiRequestClient client;

    @Autowired
    BMessageRepository bMessageRepository;


    /**
    * 保存 bDiscuss
    * @param bDiscuss
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(CurUser curUser,BDiscuss bDiscuss){
        //获取当前企业的评论通知权限
        List<BMessage> bMessageList = bMessageRepository.queryByNameAndCorp("HFPL",curUser.getCorpId());//回复评论
        BMessage message = new BMessage();
        if (bMessageList != null && bMessageList.size() > 0){
            message = bMessageList.get(0);
        }
        //发送通知
        if (StringUtils.isNotBlank(bDiscuss.getLevel()) && "2".equals(bDiscuss.getLevel()) && "1".equals(message.getFlag())){
            BNotice bNotice = new BNotice();
            if (StringUtils.isNotEmpty(bDiscuss.getReplyId())){
                BDiscuss bDiscussStu = bDiscussRepository.selectByPrimaryKey(bDiscuss.getReplyId());
                bNotice.setRecipientIds(new ArrayList<>(Arrays.asList(bDiscussStu.getStudentId())));
            }else {
                BDiscuss bDiscussStu = bDiscussRepository.selectByPrimaryKey(bDiscuss.getPid());
                bNotice.setRecipientIds(new ArrayList<>(Arrays.asList(bDiscussStu.getStudentId())));
            }

            bNotice.setInfoType(InfoTypeEnum.REPLY_DISCUSS.getValue());
            bNotice.setLessonId(bDiscuss.getLessonId());
            bNotice.setSenderId(curUser.getUserId());
            bNotice.setCorpId(curUser.getCorpId());
            // 获取课程信息
            BLesson bLesson = lessonRepository.selectByPrimaryKey(bDiscuss.getLessonId());
            if(bLesson == null){
                throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
            }
            String lessonType = LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType()) ? "线上课程" : "线下培训";
            String urlType = LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType()) ? UrlTypeEnum.LESSON.getName() : UrlTypeEnum.TRAIN.getName();
            bNotice.setInfoText(sysUserServiceImpl.getNameById(bDiscuss.getStudentId()).getName() + "回复了您在" + lessonType +"\"" + bLesson.getLessonName() +"\"的评论" );
            // 发消息
            // InfoTypeEnum.REPLY_DISCUSS.getvalue()
            client.send(bNotice,InfoTypeEnum.REPLY_DISCUSS.getValue(), MessageEnum.HFPL.getName(),urlType);
        }


        //评论课程
        List<BMessage> bMessageListFBPL = bMessageRepository.queryByNameAndCorp("FBPL",curUser.getCorpId());//发布评论
        BMessage messageFBPL = new BMessage();
        if (bMessageListFBPL != null && bMessageListFBPL.size() > 0){
            messageFBPL = bMessageListFBPL.get(0);
        }
        if (StringUtils.isNotBlank(bDiscuss.getLevel()) && "1".equals(bDiscuss.getLevel()) && "1".equals(messageFBPL.getFlag())){
            BNotice bNotice = new BNotice();
            bNotice.setInfoType(InfoTypeEnum.REPLY_DISCUSS.getValue());
            bNotice.setLessonId(bDiscuss.getLessonId());
            bNotice.setSenderId(curUser.getUserId());
            bNotice.setCorpId(curUser.getCorpId());
            // 获取课程信息
            BLesson bLesson = lessonRepository.selectByPrimaryKey(bDiscuss.getLessonId());
            if(bLesson == null){
                throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
            }
            if(StringUtils.isEmpty(bLesson.getLecturerId())){
                throw new ServiceException(ResultServiceEnums.NO_LECTURE);
            }
            bNotice.setRecipientIds(new ArrayList<>(Arrays.asList(bLesson.getLecturerId())));
            String lessonType = LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType()) ? "线上课程" : "线下培训";
            String urlType = LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType()) ? UrlTypeEnum.LESSON.getName() : UrlTypeEnum.TRAIN.getName();
            bNotice.setInfoText(sysUserServiceImpl.getNameById(bDiscuss.getStudentId()).getName() + "评论了您主讲的" + lessonType +"\"" + bLesson.getLessonName()+"\"" );
            // 发消息
            // InfoTypeEnum.REPLY_DISCUSS.getvalue()
            client.send(bNotice,InfoTypeEnum.REPLY_DISCUSS.getValue(), MessageEnum.FBPL.getName(),urlType);
        }
        //增加操作
        UserUtil.setCurrentPersistOperation(bDiscuss);
        //将评论的数据添加到数据库
        int line = bDiscussRepository.insertSelective(bDiscuss);
        //重新统计b_lesson评论数
        bLessonRepository.discussNumberReCount(bDiscuss.getLessonId());
        return new PersistModel(line);
    }


    /**
    * 逻辑删除
    * @param bDiscuss
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BDiscuss bDiscuss){
        UserUtil.setCurrentMergeOperation(bDiscuss);
        bDiscuss.setDeleted();
        BDiscuss bDiscuss1 = bDiscussRepository.selectByPrimaryKey(bDiscuss);
        int line = bDiscussRepository.logicalDelete(bDiscuss);
        //删除二级子评论
        bDiscussRepository.logicalDeleteSon(bDiscuss);
        if (bDiscuss1 != null && StringUtils.isNotEmpty(bDiscuss1.getLessonId())){
            bLessonRepository.discussNumberReCount(bDiscuss1.getLessonId());
        }
        return new PersistModel(line);
    }

    /**
    * 修改 bDiscuss
    * @param bDiscuss
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BDiscuss bDiscuss){
        UserUtil.setCurrentMergeOperation(bDiscuss);
        int line = bDiscussRepository.updateByPrimaryKeySelective(bDiscuss);
//        log.info(UserUtil.getUserInfoProp(bDiscuss.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
//            bDiscuss.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bDiscuss
    * @param id
    * @return
    */
    public BDiscuss findById(String id){
        return bDiscussRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bDiscuss
    * @param bDiscuss
    * @return
    */
    public List<BDiscuss> findAllByPagination(PageUtil<BDiscuss> paginationUtility,
        BDiscuss bDiscuss){
        bDiscuss.setStart();
        bDiscuss.setNotDeleted();
        return bDiscussRepository.queryBDiscusss(bDiscuss);
    }

    @Override
    public Map getDiscussByLessonId(String lessonId,String corpId) {

        Set<String> ids = new HashSet<>();

        List<BDiscussDto> bDiscussList = bDiscussRepository.getDiscussByLessonId(lessonId,"",corpId);
        bDiscussList.stream().forEach(bDiscuss -> {
            List<BDiscussDto> bDiscussChildList = bDiscussRepository.getDiscussByLessonId(lessonId,bDiscuss.getBusinessId(),corpId);
            bDiscuss.setBDiscussChildList(bDiscussChildList);
            bDiscuss.setDiscussDate();
            bDiscuss.setDiscussTime();
            bDiscussChildList.forEach(bDiscussChild -> {
                bDiscussChild.setDiscussDate();
                bDiscussChild.setDiscussTime();
                if (StringUtils.isNotEmpty(bDiscussChild.getStudentId())){
                    ids.add(bDiscussChild.getStudentId());
                }
            });
            if (StringUtils.isNotEmpty(bDiscuss.getStudentId())){
                ids.add(bDiscuss.getStudentId());
            }
        });

        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(corpId);
        mtUserGetsReqDTO.setIds(new ArrayList<>(ids));
        List<MTUserInfoRspDTO> mtUserInfoRspDTOList = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
        if (mtUserInfoRspDTOList != null && mtUserInfoRspDTOList.size() > 0){
            mtUserInfoRspDTOList.forEach(mtUserInfoRspDTO -> {
                bDiscussList.forEach(bDiscussDto -> {
                    if (mtUserInfoRspDTO.getId().equals(bDiscussDto.getStudentId())) {
                        //设置一级评论头像
//                        myInfoDTO.setName(mtUserInfoRspDTO.getName());
                        bDiscussDto.setHeadPic(mtUserInfoRspDTO.getAvatar());
                    }
                    bDiscussDto.getBDiscussChildList().forEach(bDiscussChild -> {
                        if (mtUserInfoRspDTO.getId().equals(bDiscussChild.getStudentId())) {
                            //设置二级评论头像
//                        myInfoDTO.setName(mtUserInfoRspDTO.getName());
                            bDiscussChild.setHeadPic(mtUserInfoRspDTO.getAvatar());
                        }
                    });
                });
            });
        }

        Map<String,Object> result = new HashMap<>();
        result.put("discuss",bDiscussList);
        BLesson bLesson = bLessonRepository.selectByPrimaryKey(lessonId);
        if (bLesson != null){
            result.put("number",bLesson.getDiscussNumber());
        }
        return result;
    }

}
