package org.rcisoft.business.blabel.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.blabel.entity.BLabel;
import org.rcisoft.business.blabel.service.BLabelService;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

@Api(tags = "05 标签")
@RestController
@RequestMapping("/BLabel")
public class BLabelController extends PaginationController<BLabel> {
    @Autowired
    private BLabelService bLabelService;

    @ApiOperation(value="501 根据条件分页查询", notes="根据条件分页查询标签", response = BLabel.class)
    @ApiImplicitParam(name = "lName", value = "(标签名)模糊查询条件", required = false, dataType = "varchar")
    @GetMapping(value = "/queryBlabelsByPagination")
    public Result queryBlabelsByPagination(CurUser curUser, @Valid String lName, BindingResult br){
        bLabelService.queryBlabelsByPagination(getPaginationUtility(),lName,curUser.getCorpId());
        GridModel gridModel = getGridModelResponse();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                gridModel);
    }

    @ApiOperation(value="502 物理删除", notes="根据ID删除一条记录")
    @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "varchar")
    @PostMapping(value = "/remove")
    public Result remove(CurUser curUser,@Valid String id,BindingResult br) {
        PersistModel data = bLabelService.removeLabel(id);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="503 更新标签名", notes="根据businessId更新标签名")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "标签id", required = true, dataType = "varchar"),
                @ApiImplicitParam(name = "lName", value = "标签名", required = true, dataType = "varchar")})
    @PostMapping(value = "/update")
    public Result update(CurUser curUser,@Valid BLabel bLabel,BindingResult br){
        PersistModel data = bLabelService.updateLabel(bLabel,curUser);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLabel);
    }

    @ApiOperation(value="504 添加新标签", notes="根据businessId插入新标签")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "标签id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "lName", value = "标签名", required = true, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result insert(CurUser curUser,@Valid BLabel bLabel,BindingResult br){
        PersistModel data = bLabelService.insertLabel(bLabel,curUser);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLabel);
    }

    @ApiOperation(value="505 根据条件查询", notes="根据条件查询标签", response = BLabel.class)
    @ApiImplicitParam(name = "lName", value = "(标签名)模糊查询条件", required = false, dataType = "varchar")
    @GetMapping(value = "/queryBlabels")
    public Result queryBlabels(CurUser curUser, @Valid String lName, BindingResult br){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bLabelService.queryBlabels(lName,curUser));
    }

}
