package org.rcisoft.business.blesson.service.impl;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.blesson.dao.BAppointRepository;
import org.rcisoft.business.blesson.dao.BHotRepository;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.dto.*;
import org.rcisoft.business.blesson.entity.BAppoint;
import org.rcisoft.business.blesson.entity.BHot;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.dao.BLessonPersonRepository;
import org.rcisoft.business.blesson.entity.BLessonPerson;
import org.rcisoft.business.blesson.enums.IsApplyEnum;
import org.rcisoft.business.blesson.enums.IsFinishEnum;
import org.rcisoft.business.blesson.enums.LessonTypeEnum;
import org.rcisoft.business.blesson.enums.ReleaseStateEnum;
import org.rcisoft.business.blesson.service.BLessonPersonService;
import org.rcisoft.business.bmessage.dao.BMessageRepository;
import org.rcisoft.business.bmessage.entity.BMessage;
import org.rcisoft.business.bpersonvalue.dao.BPersonValueRepository;
import org.rcisoft.business.bpersonvalue.entity.BPersonValue;
import org.rcisoft.common.util.feignDto.*;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.enums.MessageEnum;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.dto.MyInfoDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;


/**
 * @author
 * @date 2019/9/10
 */
@Service
@Slf4j
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class BLessonPersonServiceImpl implements BLessonPersonService {

    @Autowired
    private BLessonRepository bLessonRepository;

    @Autowired
    private BLessonPersonRepository bLessonPersonRepository;

    @Autowired
    private BAppointRepository bAppointRepository;

    @Autowired
    private BHotRepository bHotRepository;

    @Autowired
    MTCotactApiRequestClient mtCotactApiRequestClient;

    @Autowired
    SysUserMapper sysUserMapper;

    @Autowired
    BPersonValueRepository bPersonValueRepository;

    @Autowired
    BLessonServiceImpl bLessonServiceImpl;

    @Autowired
    BMessageRepository bMessageRepository;


    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public PersistModel quit(QuitAndSignDTO quitAndSignDTO, CurUser curUser) {
        BLessonPerson bLessonPerson = getLessonPersonByDto(quitAndSignDTO);

        BLesson bLesson = new BLesson();
        bLesson.setBusinessId(quitAndSignDTO.getLessonId());
        bLesson = bLessonRepository.selectByPrimaryKey(bLesson);
        if (bLesson == null){
            throw new ServiceException(ResultServiceEnums.NOT_LESSON_ERROR);
        }
        if (bLessonPerson == null && LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType())){
            throw new ServiceException(ResultServiceEnums.APPOINT_NOT_CLOSE);
        }
        if (bLessonPerson == null && LessonTypeEnum.TRAIN.getCode().equals(bLesson.getLessonType())){
            throw new ServiceException(ResultServiceEnums.LEARN_LESSON_NOT_EXISTS);
        }


        if ("1".equals(bLessonPerson.getIsAppoint())){
            throw new ServiceException(ResultServiceEnums.APPLY_TRAIN_NOT_EXISTS);
        }
        bLessonPerson.setDelFlag("1");
        //将加入学习时间置空
        bLessonPerson.setApplyDate(null);

        UserUtil.setCurrentMergeOperation(bLessonPerson);
//        int line = bLessonPersonRepository.quit(quitAndSignDTO.getBusinessId());
        int line = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
        //重新统计在学人数
        bLessonRepository.personNumberReCount(quitAndSignDTO.getLessonId(),bLesson.getLessonType());
        return new PersistModel(line);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public List<BLessonPerson> queryLearnBLessonsByPagination(PageUtil pageUtil, ILearnLessonDTO model) {
        List<BLessonPerson> bLessonList = bLessonPersonRepository.queryLearnBLessons(model);
        if (LessonTypeEnum.TRAIN.getCode().equals(model.getLessonType())){
            Date now = new Date();
            bLessonList.forEach(bLesson -> {
                Long time = Long.parseLong(bLesson.getTrainSignTime()) * 60 * 1000;
                Date signStart = new Date(bLesson.getTrainStartDate().getTime() - time);
                if (now.before(signStart)){
                    bLesson.setTrainType("0");
                }else if (bLesson.getTrainOverDate().before(now)){
                    bLesson.setTrainType("2");
                }else{
                    bLesson.setTrainType("1");
                }
            });
        }
        return addNew(bLessonList);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public StudentTrackingChartRspDTO studentTrackingChart(StudentTrackingChartDTO studentTrackingChartDTO) {
        return bLessonPersonRepository.studentTrackingChart(studentTrackingChartDTO);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public List<StudentTrackingRspDTO> studentTrackingByPagination(PageUtil pageUtil, StudentTrackingDTO studentTrackingDTO) {
//        List<Long> deptIds = new ArrayList<>();
        BLesson bLesson = bLessonRepository.selectByPrimaryKey(studentTrackingDTO.getLessonId());
        if (bLesson == null){
            throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
        }
        String trainType = "";
        if (LessonTypeEnum.TRAIN.getCode().equals(bLesson.getLessonType())){
            Date now = new Date();
            Long time = Long.parseLong(bLesson.getTrainSignTime()) * 60 * 1000;
            Date signStart = new Date(bLesson.getTrainStartDate().getTime() - time);
            if (now.before(signStart)){
                trainType = "0";
            }else if ( bLesson.getTrainOverDate().before(now)){
                trainType = "2";
            }else{
                trainType = "1";
            }
        }

        if(StringUtils.isNotEmpty(studentTrackingDTO.getDepartId())){

            Set<String> deptIds = new HashSet<>();
            deptIds.add(studentTrackingDTO.getDepartId());
            Set<String> deptSets = new HashSet<>(deptIds);
            List<String> datas = mtCotactApiRequestClient.getSubUserIdsByDeptIds(studentTrackingDTO.getCorpId(), deptSets);
//            List<UserGetStudentTrackingRspDTO> datas = mtCotactApiRequestClient.allUserListByDeptAtStuTrack(studentTrackingDTO.getCorpId(),studentTrackingDTO.getDepartId());
            if (datas != null && datas.size()>0){
                List<String> userIdsInDepart = new ArrayList<>(datas);
//                datas.forEach(data->{
//                    userIdsInDepart.addAll(data);
//                });
                studentTrackingDTO.setUserIdsInDepart(userIdsInDepart);
            }
            if (studentTrackingDTO.getUserIdsInDepart() == null || studentTrackingDTO.getUserIdsInDepart().size() < 1){
                return new ArrayList<>();
            }
        }
        List<StudentTrackingRspDTO> studentTrackingRspDTOs = bLessonPersonRepository.studentTrackingByPagination(studentTrackingDTO);
        List<String> uids = new ArrayList<>();
        if (studentTrackingRspDTOs != null && studentTrackingRspDTOs.size() > 0){
            studentTrackingRspDTOs.forEach(studentTrackingRspDTO->{
                uids.add(studentTrackingRspDTO.getPersonId());
            });
        }
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(studentTrackingDTO.getCorpId());
        mtUserGetsReqDTO.setIds(uids);
        //遍历获取部门名
        List<MTUserInfoRspDTO> departs = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
        if (studentTrackingRspDTOs != null && studentTrackingRspDTOs.size() > 0 && departs != null && departs.size() > 0){
            for (StudentTrackingRspDTO studentTrackingRspDTO : studentTrackingRspDTOs) {
                for (MTUserInfoRspDTO depart : departs) {
                    if (studentTrackingRspDTO.getPersonId().equals(depart.getId()) && depart.getDepts() != null && depart.getDepts().size() > 0){
                        studentTrackingRspDTO.setDepartName(depart.getDepts().get(0).getName());
                    }
                }
                studentTrackingRspDTO.setTrainType(trainType);
            }
        }
        return studentTrackingRspDTOs;
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public PersistModel cancelAppoint(CancelAppointDTO cancelAppointDTO) {
//        String bAppointId = bLessonPersonRepository.selectAppointIdById(cancelAppointDTO);
        BLessonPerson bLessonPerson = bLessonPersonRepository.selectById(cancelAppointDTO.getBusinessId());
        if (bLessonPerson == null || StringUtils.isEmpty(bLessonPerson.getAppointId())){
            throw new ServiceException(ResultServiceEnums.APPOINT_NOT_CANCEL);
        }

        BAppoint bAppoint = bAppointRepository.selectById(bLessonPerson.getAppointId());
        //改变bAppoint 值
        if (bAppoint == null){
            throw new ServiceException(ResultServiceEnums.NOT_APPOINT_INFO);
        }
        String designatedIds = bAppoint.getDesignatedId();
        String[] designatedIdList = designatedIds.split(",");
        String designatedIdRes = "";
        for (String s : designatedIdList) {
            if (!s.equals(cancelAppointDTO.getPersonId())){
                designatedIdRes += s + ",";
            }
        }
        //去除最后逗号
        if (designatedIdRes.length() > 0) {
            designatedIdRes = designatedIdRes.substring(0, designatedIdRes.length() - 1);
        }
        //删除b_appoint表
//        UserUtil.setCurrentMergeOperation(bAppoint);
//        bAppoint.setDesignatedId(designatedIdRes);

        //查询是否在学习
        BLesson bLesson = bLessonRepository.selectByPrimaryKey(bLessonPerson.getLessonId());
//        String isFinish = bLessonPersonRepository.selectIsFinish(cancelAppointDTO);
//        BLessonPerson bLessonPerson1 = bLessonPersonRepository.selectById(cancelAppointDTO.getBusinessId());
        int result = 0;
        //课程
        if (bLesson != null && LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType())  && IsFinishEnum.NOT_START.getCode().equals(bLessonPerson.getIsFinish())){
            //未开始执行删除
            bLessonPerson.setDelFlag("1");
            //将加入学习时间置空
            bLessonPerson.setApplyDate(null);
            bLessonPerson.setAppointId("");
            bLessonPerson.setIsAppoint("0");
            result = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
        }
        if (bLesson != null && LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType())  && (IsFinishEnum.LEARNING.getCode().equals(bLessonPerson.getIsFinish()) || IsFinishEnum.FINISHED.getCode().equals(bLessonPerson.getIsFinish()))){
            //已开始执行更新
            UserUtil.setCurrentMergeOperation(bLessonPerson);
            bLessonPerson.setIsAppoint("0");
            bLessonPerson.setAppointId("");
            result = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
        }
        //培训
        if (bLesson != null && LessonTypeEnum.TRAIN.getCode().equals(bLesson.getLessonType())){

            UserUtil.setCurrentMergeOperation(bLessonPerson);
            bLessonPerson.setIsAppoint("0");
            bLessonPerson.setAppointId("");
            result = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
        }

        //没有指派人员 删除指派表
        if (StringUtils.isEmpty(designatedIdRes)){
            bAppointRepository.deleteByPrimaryKey(bAppoint);
        }else {
            //更新b_appoint表
            UserUtil.setCurrentMergeOperation(bAppoint);
            bAppoint.setDesignatedId(designatedIdRes);
            bAppointRepository.updateByPrimaryKeySelective(bAppoint);
        }

        return new PersistModel(1,result);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public PersistModel addMyLearnLesson(LessonAndUserDTO lessonAndUserDTO, CurUser curUser) {

        List<BLesson> bLessonList = bLessonRepository.selectInfoById(lessonAndUserDTO.getLessonId(),curUser);
        //判断课程是否存在
        if (bLessonList == null || bLessonList.size() < 1 || bLessonList.get(0) == null){
            throw new ServiceException(ResultServiceEnums.LESSON_NOT_EXISTA);
        }
        BLesson bLesson = bLessonList.get(0);
        //判断课程是否被关闭
        if (ReleaseStateEnum.CLOSED.getCode().equals(bLesson.getReleaseState())){
            throw new ServiceException(ResultServiceEnums.LESSON_CLOSED);
        }
        if (!(LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType()))){
            throw new ServiceException(ResultServiceEnums.NOT_TRAIN_TYPE);
        }

        //添加至b_hot表 并重新统计热度
        int count = bHotRepository.selectCountByPersonId(lessonAndUserDTO.getUserId(), lessonAndUserDTO.getLessonId());
        if (count < 1){
            bHotRepository.insertSelective(new BHot(IdGen.uuid(), lessonAndUserDTO.getUserId(), lessonAndUserDTO.getLessonId()));
            bLessonRepository.hotNumberReCount(lessonAndUserDTO.getLessonId());
        }

        List<BLessonPerson> bLessonPersons = bLessonPersonRepository.selectByPersonAndLesson(lessonAndUserDTO);
        BLessonPerson bLessonPerson;
        int line = 0;
        //被指派未开始
        if (bLessonPersons != null && bLessonPersons.size()>0 && IsFinishEnum.NOT_START.getCode().equals(bLessonPersons.get(0).getIsFinish()) ){
            bLessonPerson = bLessonPersons.get(0);
            bLessonPerson.setIsFinish("1");
            bLessonPerson.setApplyDate(new Date());
            line = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
            //重新统计在学人数
            bLessonRepository.personNumberReCount(lessonAndUserDTO.getLessonId(),"0");
            return new PersistModel(line);
        }
        if (bLessonPersons == null || bLessonPersons.size() < 1){    //未指派
            //判断用户是否有足够积分
            MyInfoDTO userInfo =  sysUserMapper.getNameById(curUser.getUserId());
            if (userInfo == null){
                throw new ServiceException(ResultServiceEnums.COMPANY_NOT_EXISTS);
            }
            if (!(curUser.getUserId().equals(bLesson.getLecturerId())) && bLesson.getValueConsume() != null && !("0".equals(bLesson.getValueConsume())) && (StringUtils.isEmpty(userInfo.getValue()) ||  Long.valueOf(userInfo.getValue()) < Long.valueOf(bLesson.getValueConsume()))){
                throw new ServiceException(ResultServiceEnums.VALUE_NOT_ENOUGH);
            }
            //查询是否已有已删除
            List<BLessonPerson> bLessonPersonReals = bLessonPersonRepository.selectRealByPersonAndLesson(lessonAndUserDTO);
            if (bLessonPersonReals != null && bLessonPersonReals.size()>0){
                BLessonPerson bLessonPersonReal = bLessonPersonReals.get(0);
                bLessonPersonReal.setFlag("1");
                bLessonPersonReal.setDelFlag("0");
                //更新报名时间
                bLessonPersonReal.setApplyDate(new Date());

                UserUtil.setCurrentMergeOperation(bLessonPersonReal);
                line = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPersonReal);
                //扣除对应积分 添加用户积分详情 更新用户积分
                //增加用户积分详情
                if (bLesson.getValueConsume() != null && !"0".equals(bLesson.getValueConsume()) && !(curUser.getUserId().equals(bLesson.getLecturerId()))){
                    BPersonValue bPersonValue = new BPersonValue(curUser.getUserId(),"学习!@#课程",bLesson.getValueConsume(),"1",bLesson.getBusinessId());
                    bLessonServiceImpl.addValueEvent(curUser.getUserId(),bPersonValue,userInfo.getLockNum());
                }
                bLessonRepository.personNumberReCount(lessonAndUserDTO.getLessonId(),"0");
                return new PersistModel(line);
            }
            bLessonPerson = new BLessonPerson();
            bLessonPerson.initModel();
            bLessonPerson.setIsFinish("1");
            //添加报名时间
            bLessonPerson.setApplyDate(new Date());
            bLessonPerson.setLessonId(lessonAndUserDTO.getLessonId());
            bLessonPerson.setPersonId(lessonAndUserDTO.getUserId());
            line = bLessonPersonRepository.insertBLessonPerson(bLessonPerson);
            //扣除对应积分 添加用户积分详情 更新用户积分
            //增加用户积分详情
            //积分变化后 s_user表中value字段重新统计
            if (bLesson.getValueConsume() != null && !"0".equals(bLesson.getValueConsume()) && !(curUser.getUserId().equals(bLesson.getLecturerId()))){
                BPersonValue bPersonValue = new BPersonValue(curUser.getUserId(),"学习!@#课程",bLesson.getValueConsume(),"1",bLesson.getBusinessId());
                bLessonServiceImpl.addValueEvent(curUser.getUserId(),bPersonValue,userInfo.getLockNum());
            }
            //重新统计在学人数
            bLessonRepository.personNumberReCount(lessonAndUserDTO.getLessonId(),"0");
            return new PersistModel(line);
        }
//        if (bLessonPersons != null && bLessonPersons.size()>0){
//            line = 1;
//            return new PersistModel(line);
//        }

        return  new PersistModel(1);
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    @Override
    public PersistModel signIn(QuitAndSignDTO quitAndSignDTO) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        BLessonPerson bLessonPerson = getLessonPersonByDto(quitAndSignDTO);
        log.debug("=========签到1=====>>>>"+bLessonPerson);

        BLesson bLesson = new BLesson();
        bLesson.setBusinessId(quitAndSignDTO.getLessonId());
        bLesson = bLessonRepository.selectByPrimaryKey(bLesson);
        if (bLesson == null){
            throw new ServiceException(ResultServiceEnums.NOT_LESSON_ERROR);
        }
        if (bLessonPerson == null || IsApplyEnum.NOT_APPLY.getCode().equals(bLessonPerson.getIsApply())){
            throw new ServiceException(ResultServiceEnums.APPLY_TRAIN_NOT_EXISTS);
        }

        Date now = new Date();
        Long time = Long.parseLong(bLesson.getTrainSignTime()) * 60 * 1000;
        Date signStart = new Date(bLesson.getTrainStartDate().getTime() - time);
        log.debug("==========签到2========>>>>当前时间"+simpleDateFormat.format(now));
        //时间不在签到范围内
        if (now.before(signStart) || bLesson.getTrainOverDate().before(now)){
            throw new ServiceException(ResultServiceEnums.NOT_SIGN_DATE);
        }
        if ("2".equals(bLessonPerson.getTrainIsSign())){
            throw new ServiceException(ResultServiceEnums.TRAIN_SIGNED);
        }
        //LZ 判断 曾经是否学完过 奖励积分
        if ( !("2".equals(bLessonPerson.getTrainIsSign())) && !("1".equals(bLessonPerson.getEverFinished()))){
            //查询课程信息
            if(StringUtils.isNotEmpty(bLesson.getValueGain()) && Long.parseLong(bLesson.getValueGain()) > 0){
                BPersonValue bPersonValue = new BPersonValue(bLessonPerson.getPersonId(),"参加!@#培训",bLesson.getValueGain(),"0",bLesson.getBusinessId());
                bLessonServiceImpl.addValueEvent(bLessonPerson.getPersonId(),bPersonValue,null);
            }
            bLessonPerson.setEverFinished("1");
        }
        bLessonPerson.setSignDate(new Date());
        log.debug("========签到3======>>>>"+simpleDateFormat.format(bLessonPerson.getSignDate()));
        bLessonPerson.setTrainIsSign("2");
        UserUtil.setCurrentMergeOperation(bLessonPerson);
        log.debug("========签到4======>>>>"+bLessonPerson);
        int line = bLessonPersonRepository.updateByPrimaryKeySelective(bLessonPerson);
        return new PersistModel(line);
    }

    @Override
    public int selectByPersonAndLesson(LessonAndUserDTO lessonAndUserDTO) {
        List<BLessonPerson> bLessonPersonList = bLessonPersonRepository.selectByPersonAndLesson(lessonAndUserDTO);
//        BLesson bLesson = bLessonRepository.selectByPrimaryKey(lessonAndUserDTO.getLessonId());

        int result = 0;
        if (bLessonPersonList == null || bLessonPersonList.size() < 1 ){
            result = 1;
        }
//        if (bLesson != null && lessonAndUserDTO.getUserId().equals(bLesson.getLecturerId())){
//            result = 0;
//        }
        return result;
    }

    @Transactional(propagation = Propagation.REQUIRED, readOnly = false,rollbackFor= Exception.class)
    BLessonPerson getLessonPersonByDto(QuitAndSignDTO quitAndSignDTO){
        BLessonPerson bLessonPerson = null;
        if (StringUtils.isNotEmpty(quitAndSignDTO.getBusinessId())){
            bLessonPerson = bLessonPersonRepository.selectById(quitAndSignDTO.getBusinessId());
        }else {
            LessonAndUserDTO lessonAndUserDTO = new LessonAndUserDTO(quitAndSignDTO.getLessonId(), quitAndSignDTO.getCorpId(), quitAndSignDTO.getUserId());
            List<BLessonPerson> bLessonPersonList = bLessonPersonRepository.selectByPersonAndLesson(lessonAndUserDTO);
            if (bLessonPersonList != null && bLessonPersonList.size() > 0 && bLessonPersonList.get(0) != null){
                bLessonPerson = bLessonPersonList.get(0);
            }
        }
        return bLessonPerson;
    }

    /**
     * 新建的课程培训 添加new字段
     *
     * @param result
     * @return
     */
    public List<BLessonPerson> addNew(List<BLessonPerson> result) {
        if(result == null)
            return result;
        //获取当前登录人信息
        CurUser curUser = UserUtil.getCurUser();
        String userId = curUser.getUserId();
        String corpId = curUser.getCorpId();
        log.info("-----------addNew---------当前登录人信息curUser----------"+curUser+"--------------------");
        BMessage bMessage = new BMessage();
        bMessage.setName(MessageEnum.NEWNOTICE.getName());
        //查询当前企业的 新消息时间 的配置
        List<BMessage> messageList = bMessageRepository.queryByNameAndCorp(MessageEnum.NEWNOTICE.getName(),corpId);
        log.info("-----------addNew---------新消息配置messageList---------"+messageList+"--------------");
        if (messageList.size() <= 0)
            return result;
        BMessage newNotice = messageList.get(0);
        //获取当前时间
        Calendar c = Calendar.getInstance();
        int time = Integer.parseInt(newNotice.getTimeValue());
        for (BLessonPerson b : result) {
            //step 1 查询look表中是否有对应任何课程的点击记录
            int flag = bLessonRepository.getCountByUserAndLesson(userId, b.getBusinessId(), "0");
            //step 2 存在则返回，不存在则判断时间 满足条件 setIsNew() flag <= 0 说明此用户未点击过该课程
            if (flag <= 0) {
                //获取课程的 指派时间
                Calendar c1 = DateUtil.calendar(b.getCreateDate());
                log.info("-----------addNew------课程指派时间--"+DateUtil.date(c1)+"---------------");
                switch (newNotice.getValueType()){
                    case "1" : c1.add(Calendar.HOUR, time); break;
                    case "2" : c1.add(Calendar.DAY_OF_MONTH, time);  break;
                    case "3" : c1.add(Calendar.WEEK_OF_MONTH, time); break;
                    case "4" : c1.add(Calendar.MONTH, time); break;
                }
                log.info("-----------addNew-------当前时间--"+DateUtil.date(c)+"--new标志结束时间--"+DateUtil.date(c1));
                if (c1.after(c)){
                    b.setIsNew("1");
                    log.info("-----------addNew---------当前课程已设置new-------"+b);
                }
            }
        }
        return result;
    }

}
