package org.rcisoft.business.blesson.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.blesson.dto.AppointLessonDTO;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.enums.LessonTypeEnum;
import org.rcisoft.business.blesson.service.SendNoticeService;
import org.rcisoft.business.bmessage.dao.BMessageRepository;
import org.rcisoft.business.bmessage.entity.BMessage;
import org.rcisoft.business.bnotice.entity.BNotice;
import org.rcisoft.common.model.InfoTypeEnum;
import org.rcisoft.common.util.outClient.MTNotificationApiRequestClient;
import org.rcisoft.core.enums.MessageEnum;
import org.rcisoft.core.enums.UrlTypeEnum;
import org.rcisoft.sys.user.service.impl.SysUserServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Slf4j
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class SendNoticeServiceImpl implements SendNoticeService {

    @Autowired
    BMessageRepository bMessageRepository;

    @Autowired
    SysUserServiceImpl sysUserServiceImpl;

    @Autowired
    MTNotificationApiRequestClient client;

    @Override
    public void sendAppointNotice(BLesson bLesson,AppointLessonDTO appointLessonDTO, List<String> userIds ) {

        //是否去掉已在切已指派的人员
        String type = LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType()) ? MessageEnum.ZPKC.getName() : MessageEnum.ZPPX.getName();
        List<BMessage> bMessageList = bMessageRepository.queryByNameAndCorp(type,appointLessonDTO.getCorpId());
        BMessage message = new BMessage();
        if (bMessageList != null && bMessageList.size() > 0){
            message = bMessageList.get(0);
        }
        if (userIds != null  && userIds.size() > 0 && message != null && "1".equals(message.getFlag())){
            BNotice bNotice = new BNotice();
            bNotice.setRecipientIds(userIds);
            bNotice.setLessonId(bLesson.getBusinessId());
            bNotice.setSenderId(appointLessonDTO.getUserId());
            bNotice.setCorpId(appointLessonDTO.getCorpId());
            // 获取课程信息
            String lessonType = LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType()) ? "线上课程" : "线下培训";
            String infoType = LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType()) ? InfoTypeEnum.ONLINE_LESSON.getValue() : InfoTypeEnum.OFFLINE_TRAIN.getValue();
            bNotice.setInfoType(infoType);
            String urlType = LessonTypeEnum.LESSON.getCode().equals(bLesson.getLessonType()) ? UrlTypeEnum.LESSON.getName() : UrlTypeEnum.TRAIN.getName();
            bNotice.setInfoText(sysUserServiceImpl.getNameById(appointLessonDTO.getUserId()).getName() + "给您指派了" + lessonType +"\"" + bLesson.getLessonName() +"\"，请及时学习" );
            // 往智信平台发啊消息
            client.send(bNotice,InfoTypeEnum.REPLY_DISCUSS.getLable(),type,urlType);
        }
    }
}
