package org.rcisoft.business.bmaterial.service.impl;

import org.apache.commons.lang.StringUtils;
import org.rcisoft.business.bmaterial.dto.DelMoreDTO;
import org.rcisoft.business.bmaterial.dto.FindDTO;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bmaterial.dao.BMaterialRepository;
import org.rcisoft.business.bmaterial.entity.BMaterial;
import org.rcisoft.business.bmaterial.service.BMaterialService;


import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import lombok.extern.slf4j.Slf4j;

import static java.util.Arrays.asList;

/**
 * Created by  on 2019-10-12 10:38:31.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BMaterialServiceImpl implements BMaterialService {

    @Autowired
    private BMaterialRepository bMaterialRepository;


    /**
    * 保存 bMaterial
    * @param bMaterial
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BMaterial bMaterial,CurUser curUser){
        if ( "0".equals(bMaterial.getFileSize())){
            throw new ServiceException(ResultServiceEnums.FILE_NOT_NULL);
        }
        List<String> urls = new ArrayList<>();
        urls.add(bMaterial.getFileUrl());
        List<BMaterial> bMaterialList = bMaterialRepository.selectByUrl(urls,"0",curUser);
        if (bMaterialList != null && bMaterialList.size() > 0){
            throw new ServiceException(ResultServiceEnums.SAVED_THIS_FILE);
        }
        //增加操作
        UserUtil.setCurrentPersistOperation(bMaterial);
        //设置上传类型为 保存
        bMaterial.setUploadType("0");
        int line = bMaterialRepository.insertMaterial(bMaterial);
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param bMaterial
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BMaterial bMaterial){
        UserUtil.setCurrentMergeOperation(bMaterial);
        bMaterial.setDeleted();
        int line = bMaterialRepository.logicalDelete(bMaterial);
        return new PersistModel(line);
    }

    @Override
    public PersistModel removeMore(String ids, CurUser curUser) {
        if (StringUtils.isEmpty(ids)){
            throw new ServiceException(ResultServiceEnums.NO_DATA);
        }
        DelMoreDTO delMoreDTO = new DelMoreDTO();
        delMoreDTO.setDelFlag("1");
        delMoreDTO.setUpdateBy(curUser.getUserId());
        delMoreDTO.setUpdateDate(new Date());
        delMoreDTO.setIdList(asList(StringUtils.split(ids, ",")));
        int line = bMaterialRepository.deleteMore(delMoreDTO);
        return new PersistModel(line);
    }

    /**
    * 修改 bMaterial
    * @param bMaterial
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BMaterial bMaterial){
        UserUtil.setCurrentMergeOperation(bMaterial);
        int line = bMaterialRepository.updateByPrimaryKeySelective(bMaterial);
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bMaterial
    * @param id
    * @return
    */
    public BMaterial findById(String id){
        return bMaterialRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bMaterial
    * @param bMaterial
    * @return
    */
    public List<BMaterial> findAllByPagination(PageUtil<BMaterial> paginationUtility,
        BMaterial bMaterial){
        bMaterial.setStart();
        bMaterial.setNotDeleted();
        return bMaterialRepository.queryBMaterials(bMaterial);
    }

    /**
    * 分页查询 bMaterial(我的资料)
    * @param findDTO
    * @return
    */
    public List<BMaterial> queryMyBMaterialByPagination(PageUtil<BMaterial> paginationUtility,
        FindDTO findDTO){
        return bMaterialRepository.queryMyBMaterialByPagination(findDTO);
    }
}
