package org.rcisoft.business.breleasevalue.service.impl;

import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.blesson.enums.LessonTypeEnum;
import org.rcisoft.business.blesson.enums.ReleaseStateEnum;
import org.rcisoft.business.breleasevalue.dto.*;
import org.rcisoft.business.breleasevalue.entity.BReleaseValueSon;
import org.rcisoft.common.util.feignDto.MTUserGetsReqDTO;
import org.rcisoft.common.util.feignDto.MTUserInfoRspDTO;
import org.rcisoft.common.util.outClient.MTCotactApiRequestClient;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.breleasevalue.dao.BReleaseValueRepository;
import org.rcisoft.business.breleasevalue.entity.BReleaseValue;
import org.rcisoft.business.breleasevalue.service.BReleaseValueService;


import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.*;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2019-11-13 13:28:38.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BReleaseValueServiceImpl implements BReleaseValueService {

    @Autowired
    private BReleaseValueRepository bReleaseValueRepository;

    @Autowired
    MTCotactApiRequestClient mtCotactApiRequestClient;

    @Override
    public PersistModel save(BReleaseValueDto bReleaseValueDto) {

        try{
            Long.parseLong(bReleaseValueDto.getLessonValue());
            Long.parseLong(bReleaseValueDto.getTrainValue());
        }catch (Exception e){
            throw new ServiceException(ResultServiceEnums.NUM_IS_ERROR);
        }

        if(bReleaseValueDto.getBusinessId() != null){
// 编辑
            BReleaseValue bReleaseValue = new BReleaseValue();
//            BeanUtils.copyProperties(bReleaseValueDto, bReleaseValue);
            bReleaseValue.setBusinessId(bReleaseValueDto.getBusinessId());
            bReleaseValue.setTitle(bReleaseValueDto.getTitle());
            bReleaseValue.setTrainValue(bReleaseValueDto.getTrainValue());
            bReleaseValue.setLessonValue(bReleaseValueDto.getLessonValue());
            UserUtil.setCurrentMergeOperation(bReleaseValue);
            int line = bReleaseValueRepository.updateByPrimaryKeySelective(bReleaseValue);
            // 删除子表中的相关记录
            bReleaseValueRepository.deleteSonForReleaseId(bReleaseValueDto.getBusinessId());
            if(StringUtils.isNotEmpty(bReleaseValueDto.getUserIds()) || StringUtils.isNotEmpty(bReleaseValueDto.getDeptIds())){
                this.splitUser(bReleaseValueDto.getUserIds(),bReleaseValueDto.getDeptIds(),bReleaseValueDto.getBusinessId());
            }
            return new PersistModel(line);
        }
//新增
        BReleaseValue bReleaseValue = new BReleaseValue();
//        BeanUtils.copyProperties(bReleaseValueDto, bReleaseValue);
        bReleaseValue.setBusinessId(bReleaseValueDto.getBusinessId());
        bReleaseValue.setTitle(bReleaseValueDto.getTitle());
        bReleaseValue.setTrainValue(bReleaseValueDto.getTrainValue());
        bReleaseValue.setLessonValue(bReleaseValueDto.getLessonValue());
        UserUtil.setCurrentPersistOperation(bReleaseValue);

        if(StringUtils.isNotEmpty(bReleaseValueDto.getUserIds()) || StringUtils.isNotEmpty(bReleaseValueDto.getDeptIds())){
            this.splitUser(bReleaseValueDto.getUserIds(),bReleaseValueDto.getDeptIds(),bReleaseValue.getBusinessId());
        }
        int line = bReleaseValueRepository.insert(bReleaseValue);
        return new PersistModel(line);
    }

    @Override
    public ReleaseValueResDto setReleaseValueResDtoIds(ReleaseValueResDto dto) {
        if(dto.getName() != null && !"".equals(dto.getName())){
            List<String> names = new ArrayList<>() ;
            if(dto.getName().contains(",")){
                names = Arrays.asList(dto.getName().split(","));
            }else if(dto.getName().contains("，")){
                names = Arrays.asList(dto.getName().split("，"));
            }else{
                names.add(dto.getName());
            }

            List<String> ids  = bReleaseValueRepository.getReleaseId(names,dto.getCorpId());
            if(ids.size() <= 0){
                List<String> list = new ArrayList<>();
                list.add("");
                dto.setIds(list);
            }else{
                dto.setIds(ids);
            }
        }
        return dto;
    }

    @Override
    public List<BReleaseValue> queryByPagination(PageUtil pageUtil, ReleaseValueResDto dto) {
        List<BReleaseValue> list = bReleaseValueRepository.queryBReleaseValues(dto);
        return list;
    }

    @Override
    public PersistModel delete(String businessId ,String corpId) {
//        删除发布积分表中的数据 del_flag = 1
        int line = bReleaseValueRepository.remove(businessId,corpId) ;
//        删除发布积分子表中与ReleaseId有关的数据
        bReleaseValueRepository.deleteSonForReleaseId(businessId);
        return new PersistModel(line);
    }

    @Override
    public List<LessonValueDto> queryLessonValueByPagination(PageUtil pageUtil, CurUser curUser, LessonValueSettingQueryDto dto) {
        try{
            if(dto.getValueConsumeEnd()!=null && !"".equals(dto.getValueConsumeEnd())){
                Long.parseLong(dto.getValueConsumeEnd());
            }
            if(dto.getValueConsumeStart()!=null && !"".equals(dto.getValueConsumeStart())){
                Long.parseLong(dto.getValueConsumeStart());
            }
            if(dto.getValueGainEnd()!=null && !"".equals(dto.getValueGainEnd())){
                Long.parseLong(dto.getValueGainEnd());
            }
            if(dto.getValueGainStart()!=null && !"".equals(dto.getValueGainStart())){
                Long.parseLong(dto.getValueGainStart());
            }
        }catch (Exception e){
            throw  new ServiceException(ResultServiceEnums.NUM_IS_ERROR);
        }
        if(dto.getTrainStatus() != null && !"".equals(dto.getTrainStatus())){
            String[] trainStatus = dto.getTrainStatus().split(",");
            StringBuffer sb = new StringBuffer(" 1 != 1");
            for(String str : trainStatus){
                //012改567
                if("5".equals(str)){
                    sb.append("  or bl.train_start_date > now()");
                }if("6".equals(str)){
                    sb.append(" or now() BETWEEN bl.train_start_date and bl.train_over_date");
                }if("7".equals(str)){
                    sb.append(" or bl.train_over_date < now() ");
                }
            }
            dto.setTrainStatus(sb.toString());
        }else {
            dto.setTrainStatus(null);
        }

        if(StringUtils.isEmpty(dto.getStatus())&&StringUtils.isEmpty(dto.getTrainStatus())){
            dto.setTrainStatus(null);
        }else if(StringUtils.isNotEmpty(dto.getStatus()) && dto.getStatus().contains("2") && StringUtils.isNotEmpty(dto.getTrainStatus())){
            dto.setTrainStatus("and (bl.release_state in("+dto.getStatus().replace("2","8")+") or (("+dto.getTrainStatus()+") and bl.release_state = '2'))");
        }else if(StringUtils.isNotEmpty(dto.getStatus()) && StringUtils.isEmpty(dto.getTrainStatus())){
            dto.setTrainStatus("and bl.release_state in("+dto.getStatus()+")");
        }


        List<LessonValueDto> list = bReleaseValueRepository.getLessonValue(curUser,dto);
        for(LessonValueDto valueDto :list){
            //0未发布 1审核 2已发布 3驳回 4关闭
            if(ReleaseStateEnum.UNRELEASED.getCode().equals(valueDto.getReleaseState())){
                valueDto.setReleaseState("未发布");
            }else if(ReleaseStateEnum.RELEASED.getCode().equals(valueDto.getReleaseState())){
                if(LessonTypeEnum.LESSON.getCode().equals(dto.getType())){
                    valueDto.setReleaseState("已发布");
                }else{
                    if(valueDto.getTrainStart().after(new Date())){
                        valueDto.setReleaseState("已发布(未开始)");
                    }else if(valueDto.getTrainStart().before(new Date()) && valueDto.getTrainEnd().after(new Date())){
                        valueDto.setReleaseState("已发布(进行中)");
                    }else if(valueDto.getTrainEnd().before(new Date())){
                        valueDto.setReleaseState("已发布(已结束)");
                    }
                }
            }else if(ReleaseStateEnum.CLOSED.getCode().equals(valueDto.getReleaseState())){
                valueDto.setReleaseState("已关闭");
            }
        }
        return list;
    }

    @Override
    public PersistModel updateLessonValue(String businessId, String valueConsume, String valueGain) {
        if(valueConsume==null || "".equals(valueConsume) || valueGain == null || "".equals(valueGain)){
            throw new ServiceException(ResultServiceEnums.VALUE_NOT_NULL);
        }
        try{
            Long.parseLong(valueConsume);
            Long.parseLong(valueGain);
        }catch (Exception e){
            throw new ServiceException(ResultServiceEnums.NUM_IS_ERROR);
        }
        int line = bReleaseValueRepository.updateLessonValue(businessId, valueConsume, valueGain);
        return new PersistModel(line);
    }

    @Override
    public List<UserValueDto> queryUserValueByPagination(PageUtil pageUtil, UserValueResDto userValueResDto) {
        List<String> deptList = new ArrayList<>();
        if(userValueResDto.getDept()!=null && !"".equals(userValueResDto.getDept())){
            Set<String> deptSets = new HashSet<>();
            deptSets.add(userValueResDto.getDept());
            deptList= mtCotactApiRequestClient.getSubUserIdsByDeptIds(userValueResDto.getCorpId(), deptSets);
            if (deptList == null || deptList.size() < 1){
                return new ArrayList<>();
            }
        }

        List<UserValueDto> users = bReleaseValueRepository.getUserValue(userValueResDto, deptList);

        List<String> list = new ArrayList<>();
        for (UserValueDto user : users) {
            list.add(user.getBusinessId());
        }
        MTUserGetsReqDTO mtUserGetsReqDTO = new MTUserGetsReqDTO();
        mtUserGetsReqDTO.setCorpId(userValueResDto.getCorpId());
        mtUserGetsReqDTO.setIds(list);
        List<MTUserInfoRspDTO> depart = mtCotactApiRequestClient.userGets(mtUserGetsReqDTO);
        if (depart != null) {
            StringBuffer sb = new StringBuffer();
            for (UserValueDto dto : users) {
                for (MTUserInfoRspDTO mtUserInfoRspDTO : depart) {
                    if (dto.getBusinessId().equals(mtUserInfoRspDTO.getId())) {
                        if (mtUserInfoRspDTO.getDepts() != null) {
                            for (MTUserInfoRspDTO.DeptsBean deptsBean : mtUserInfoRspDTO.getDepts()) {
                                for (MTUserInfoRspDTO.DeptsBean.PathBean pathBean : deptsBean.getPath()) {
                                    if (null != pathBean)
                                        sb.append(pathBean.getName() + "-");
                                }
                            }
                        }
                        if (!sb.toString().equals("")) {
                            String string = sb.toString().substring(0, sb.toString().length() - 1);
                            if(string.indexOf("-") != -1){
                                string = string.substring(string.indexOf("-")+1);
                            }
                            dto.setDept(string);
                        }
                        sb = new StringBuffer();
                    }
                }
            }
        }
        return users;
    }

    @Override
    public Map<String,Object> queryPersonReleaseValue(CurUser curUser) {
        List<BReleaseValue> list = bReleaseValueRepository.selectReleaseValueByPersonId(curUser);
        Map<String,Object> map = new HashedMap();
        if(list.size() < 1){
            map.put("trainValue","");
            map.put("lessonValue","");
            map.put("userId",curUser.getUserId());
            return map;
        }
        map.put("trainValue",list.get(0).getTrainValue());
        map.put("lessonValue",list.get(0).getLessonValue());
        map.put("userId",curUser.getUserId());
        return map;
    }

    /**
     * 去掉子表中已有用户信息，插入新的数据
     * @param userIds
     * @param deptIds
     * @param businessId
     */
    public void splitUser(String userIds ,String deptIds , String businessId){
        List<String> ids = new ArrayList<>();
        if(StringUtils.isNotEmpty(userIds)){
            ids.addAll(Arrays.asList(userIds.split(",")));
        }
        if(StringUtils.isNotEmpty(deptIds)){
            List<String> deptList = Arrays.asList(deptIds.split(","));
            Set<String> deptSets = new HashSet<>(deptList);
            List<String> data = mtCotactApiRequestClient.getSubUserIdsByDeptIds(UserUtil.getCurUser().getCorpId(), deptSets);
            ids.addAll(data);
        }
        Set<String> idSet = new HashSet<>(ids);
        // 删除子表中与当前选中用户相关的信息
        bReleaseValueRepository.deleteSonForUserId(idSet);
        //将关联信息插入子表
        List<BReleaseValueSon> list = new ArrayList<>();
        if(idSet.size() > 0){
            for (String id : idSet){
                BReleaseValueSon bReleaseValueSon = new BReleaseValueSon();
                bReleaseValueSon.setBusinessId(IdGen.uuid());
                bReleaseValueSon.setReleaseId(businessId);
                bReleaseValueSon.setUserId(id);
                list.add(bReleaseValueSon);
            }
            bReleaseValueRepository.insertIntoSon(list);
        }
    }
}
