package org.rcisoft.core.task;

//通知定时任务

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.dto.FindLessonAtMessageTaskDTO;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.bmessage.dao.BMessageRepository;
import org.rcisoft.business.bmessage.entity.BMessage;
import org.rcisoft.business.bnotice.entity.BNotice;
import org.rcisoft.business.bnotice.service.BNoticeService;
import org.rcisoft.common.model.InfoTypeEnum;
import org.rcisoft.common.util.feignDto.MTNotificationSendReqDTO;
import org.rcisoft.common.util.outClient.MTNotificationApiRequestClient;
import org.rcisoft.core.enums.MessageEnum;
import org.rcisoft.core.enums.UrlTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Component
@ConditionalOnProperty(
        prefix = "eduJob",
        name = {"messageTask"},
        havingValue = "true"
)
@Slf4j
public class MessageTask {

    @Autowired
    BMessageRepository bMessageRepository;

    @Autowired
    BLessonRepository bLessonRepository;

    @Autowired
    MTNotificationApiRequestClient client;

    @Autowired
    BNoticeService bNoticeServiceImpl;

    @Scheduled(cron = "0 0/1 * * * ?")
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public void work() throws Exception {
        Date nowDate = new Date();
        //判断是否开启
        List<BMessage> bMessagePXJJKSList = bMessageRepository.queryByNameAndCorp("PXJJKS",null);
        //判断是否在设定时间内
        if (bMessagePXJJKSList != null && bMessagePXJJKSList.size() > 0){
            bMessagePXJJKSList.forEach(bMessage -> {
                if (bMessage != null && "1".equals(bMessage.getFlag())){
                    String valueTypeStr = "MINUTE";
                    if ("2".equals(bMessage.getValueType())){
                        valueTypeStr = "HOUR";
                    }
                    if ("3".equals(bMessage.getValueType())){
                        valueTypeStr = "DAY";
                    }
                    FindLessonAtMessageTaskDTO param = new FindLessonAtMessageTaskDTO(bMessage.getCorpId(),bMessage.getTimeValue(),valueTypeStr,nowDate);
                    //在时间内发送通知
                    List<BLesson> bLessonList = bLessonRepository.selectLessonAtMessageTask(param);
                    if (bLessonList != null && bLessonList.size() > 0){
                        bLessonList.forEach(bLesson -> {
                            BNotice bNotice = new BNotice();
                            bNotice.setRecipientIds(bLesson.getPersonList());
                            bNotice.setInfoType(InfoTypeEnum.OFFLINE_TRAIN.getValue());
                            bNotice.setLessonId(bLesson.getBusinessId());
                            bNotice.setCorpId(bLesson.getCorpId());
                            // 通知信息
                            bNotice.setInfoText("\""+bLesson.getLessonName()+"\"线下培训即将开始" );
                            // 往智信平台发啊消息
                            client.send(bNotice,InfoTypeEnum.OFFLINE_TRAIN.getLable(),MessageEnum.PXJJKS.getName(),UrlTypeEnum.TRAIN.getName());
                        });
                    }
                }
            });
        }

    }
}
